@@ -9,8 +9,10 @@ from pathlib import Path
 
 from fastled.client_server import run_client_server
 from fastled.compile_server import CompileServer
+from fastled.emoji_util import EMO
 from fastled.filewatcher import file_watcher_set
 from fastled.parse_args import Args, parse_args
+from fastled.settings import DEFAULT_URL
 from fastled.sketch import find_sketch_directories, looks_like_fastled_repo
 
 
@@ -51,6 +53,41 @@ def main() -> int:
     from fastled.select_sketch_directory import select_sketch_directory
 
     args = parse_args()
+
+    if args.emsdk_headers:
+        import httpx
+
+        out_path = args.emsdk_headers
+        base_url = args.web if isinstance(args.web, str) else DEFAULT_URL
+        try:
+            response = httpx.get(f"{base_url}/headers/emsdk")
+            if response.status_code == 200:
+                Path(out_path).parent.mkdir(parents=True, exist_ok=True)
+                with open(out_path, "wb") as f:
+                    f.write(response.content)
+                print(f"{EMO('✅','SUCCESS:')} EMSDK headers exported to {out_path}")
+                return 0
+            else:
+                print(
+                    f"{EMO('❌','ERROR:')} Failed to export EMSDK headers: HTTP {response.status_code}"
+                )
+                return 1
+        except KeyboardInterrupt:
+            print("\nExiting from main...")
+            return 1
+        except Exception as e:
+            print(f"{EMO('❌','ERROR:')} Exception: {e}")
+            return 1
+
+    # Handle --install command early
+    if args.install:
+        from fastled.install.main import fastled_install
+
+        result = fastled_install(
+            dry_run=args.dry_run, no_interactive=args.no_interactive
+        )
+        return 0 if result else 1
+
     interactive: bool = args.interactive
     has_server = args.server
     update: bool = args.update
@@ -69,7 +106,7 @@ def main() -> int:
     # Check if Playwright browsers are installed
     playwright_dir = Path.home() / ".fastled" / "playwright"
     if playwright_dir.exists() and any(playwright_dir.iterdir()):
-        print(f"🎭 Playwright browsers available at: {playwright_dir}")
+        print(f"{EMO('🎭', '*')} Playwright browsers available at: {playwright_dir}")
 
     # Resolve some of the last interactive arguments
     # 1. If interactive is set and the sketch directory is not given,
