@@ -10,6 +10,7 @@ from pathlib import Path
 
 from fastled.compile_server import CompileServer
 from fastled.docker_manager import DockerManager
+from fastled.emoji_util import EMO
 from fastled.filewatcher import DebouncedFileWatcherProcess, FileWatcherProcess
 from fastled.find_good_connection import ConnectionResult
 from fastled.keyboard import SpaceBarWatcher
@@ -100,7 +101,7 @@ def _run_web_compiler(
 
     # Guard: libfastled compilation requires volume source mapping
     if not allow_libcompile:
-        print("⚠️  libfastled compilation disabled: not running in FastLED repository")
+        print(f"{EMO('⚠️', 'WARNING:')}  libfastled compilation disabled.")
 
     start = time.time()
     web_result = web_compile(
@@ -122,23 +123,8 @@ def _run_web_compiler(
         (output_dir / "index.html").write_text(error_html, encoding="utf-8")
         return web_result
 
-    def print_results() -> None:
-        hash_value = (
-            web_result.hash_value
-            if web_result.hash_value is not None
-            else "NO HASH VALUE"
-        )
-        print(
-            f"\nWeb compilation successful\n  Time: {diff:.2f}\n  output: {output_dir}\n  hash: {hash_value}\n  zip size: {len(web_result.zip_bytes)} bytes"
-        )
-
-    # now check to see if the hash value is the same as the last hash value
-    if last_hash_value is not None and last_hash_value == web_result.hash_value:
-        print("\nSkipping redeploy: No significant changes found.")
-        print_results()
-        return web_result
-
     # Extract zip contents to fastled_js directory
+    extraction_start_time = time.time()
     output_dir.mkdir(exist_ok=True)
     with tempfile.TemporaryDirectory() as temp_dir:
         temp_path = Path(temp_dir)
@@ -151,6 +137,52 @@ def _run_web_compiler(
 
         # Extract zip contents
         shutil.unpack_archive(temp_zip, output_dir, "zip")
+    extraction_time = time.time() - extraction_start_time
+
+    def print_results() -> None:
+        hash_value = (
+            web_result.hash_value
+            if web_result.hash_value is not None
+            else "NO HASH VALUE"
+        )
+
+        # Build timing breakdown
+        timing_breakdown = f"  Time: {diff:.2f} (seconds)"
+        if hasattr(web_result, "zip_time"):
+            timing_breakdown += f"\n     zip creation: {web_result.zip_time:.2f}"
+            if web_result.libfastled_time > 0:
+                timing_breakdown += (
+                    f"\n     libfastled: {web_result.libfastled_time:.2f}"
+                )
+            timing_breakdown += (
+                f"\n     sketch compile + link: {web_result.sketch_time:.2f}"
+            )
+            if hasattr(web_result, "response_processing_time"):
+                timing_breakdown += f"\n     response processing: {web_result.response_processing_time:.2f}"
+
+            # Calculate any unaccounted time
+            accounted_time = (
+                web_result.zip_time
+                + web_result.libfastled_time
+                + web_result.sketch_time
+                + web_result.response_processing_time
+                + extraction_time
+            )
+            unaccounted_time = diff - accounted_time
+            if extraction_time > 0.01:
+                timing_breakdown += f"\n     extraction: {extraction_time:.2f}"
+            if unaccounted_time > 0.01:
+                timing_breakdown += f"\n     other overhead: {unaccounted_time:.2f}"
+
+        print(
+            f"\nWeb compilation successful\n{timing_breakdown}\n  output: {output_dir}\n  hash: {hash_value}\n  zip size: {len(web_result.zip_bytes)} bytes"
+        )
+
+    # now check to see if the hash value is the same as the last hash value
+    if last_hash_value is not None and last_hash_value == web_result.hash_value:
+        print("\nSkipping redeploy: No significant changes found.")
+        print_results()
+        return web_result
 
     _chunked_print(web_result.stdout)
     print_results()
@@ -243,14 +275,41 @@ def _try_make_compile_server(
         return None
 
 
-def _is_local_host(host: str) -> bool:
+def _background_update_docker_image() -> None:
+    """Perform docker image update in the background silently."""
+    try:
+        # Only attempt update if Docker is installed and running
+        if not DockerManager.is_docker_installed():
+            return
+
+        docker_manager = DockerManager()
+        docker_running, _ = docker_manager.is_running()
+        if not docker_running:
+            return
+
+        # Silently update the docker image
+        docker_manager.validate_or_download_image(
+            image_name=IMAGE_NAME, tag="latest", upgrade=True
+        )
+    except KeyboardInterrupt:
+        import _thread
+
+        _thread.interrupt_main()
+    except Exception as e:
+        # Log warning but don't disrupt user experience
+        import warnings
+
+        warnings.warn(f"Background docker image update failed: {e}")
+
+
+def _is_local_host(url: str) -> bool:
     return (
-        host.startswith("http://localhost")
-        or host.startswith("http://127.0.0.1")
-        or host.startswith("http://0.0.0.0")
-        or host.startswith("http://[::]")
-        or host.startswith("http://[::1]")
-        or host.startswith("http://[::ffff:127.0.0.1]")
+        url.startswith("http://localhost")
+        or url.startswith("http://127.0.0.1")
+        or url.startswith("http://0.0.0.0")
+        or url.startswith("http://[::]")
+        or url.startswith("http://[::1]")
+        or url.startswith("http://[::ffff:127.0.0.1]")
     )
 
 
@@ -268,6 +327,7 @@ def run_client(
     clear: bool = False,
     no_platformio: bool = False,
     app: bool = False,  # Use app-like browser experience
+    background_update: bool = False,
 ) -> int:
     has_checked_newer_version_yet = False
     compile_server: CompileServer | None = None
@@ -312,7 +372,7 @@ def run_client(
     # Auto-detect libcompile capability on first call
     from fastled.sketch import looks_like_fastled_repo
 
-    allow_libcompile = looks_like_fastled_repo(Path(".").resolve())
+    allow_libcompile = is_local_host and looks_like_fastled_repo(Path(".").resolve())
 
     try:
 
@@ -358,11 +418,14 @@ def run_client(
                 app=app,
             )
         else:
-            print("\nCompilation successful.")
+            if result.success:
+                print("\nCompilation successful.")
+            else:
+                print("\nCompilation failed.")
             if compile_server:
                 print("Shutting down compile server...")
                 compile_server.stop()
-            return 0
+            return 0 if result.success else 1
 
         if not keep_running or shutdown.is_set():
             if http_proc:
@@ -426,9 +489,20 @@ def run_client(
                     )
                     if has_update:
                         print(f"\n🔄 {message}")
-                        print(
-                            "Run with `fastled -u` to update the docker image to the latest version."
-                        )
+                        if background_update:
+                            # Start background update in a separate thread
+                            update_thread = threading.Thread(
+                                target=_background_update_docker_image, daemon=True
+                            )
+                            update_thread.start()
+                            background_update = False
+                        else:
+                            print(
+                                "Run with `fastled -u` to update the docker image to the latest version."
+                            )
+                            print(
+                                "Or use `--background-update` to update automatically in the background after compilation."
+                            )
                 except Exception as e:
                     # Don't let Docker check failures interrupt the main flow
                     warnings.warn(f"Failed to check for Docker image updates: {e}")
@@ -461,9 +535,6 @@ def run_client(
                     print(f"\nChanges detected in FastLED source code: {changed_files}")
                     print("Press space bar to trigger compile.")
 
-                    # Re-evaluate libcompile capability when source code changes
-                    allow_libcompile = True
-
                     while True:
                         space_bar_pressed = SpaceBarWatcher.watch_space_bar_pressed(
                             timeout=1.0
@@ -477,6 +548,8 @@ def run_client(
                             print(
                                 f"Changes detected in {file_changes}\nHit the space bar to trigger compile."
                             )
+                            # Re-evaluate libcompile capability when source code changes
+                            allow_libcompile = True
 
                         if space_bar_pressed or sketch_files_changed:
                             if space_bar_pressed:
@@ -488,7 +561,9 @@ def run_client(
                             last_compiled_result = compile_function(
                                 last_hash_value=None, allow_libcompile=allow_libcompile
                             )
-                            allow_libcompile = False
+                            allow_libcompile = (
+                                allow_libcompile and not last_compiled_result.success
+                            )
                             print("Finished recompile.")
                             # Drain the space bar queue
                             SpaceBarWatcher.watch_space_bar_pressed()
@@ -513,6 +588,7 @@ def run_client_server(args: Args) -> int:
     profile = bool(args.profile)
     web: str | bool = args.web if isinstance(args.web, str) else bool(args.web)
     auto_update = bool(args.auto_update)
+    background_update = bool(args.background_update)
     localhost = bool(args.localhost)
     directory = args.directory if args.directory else Path(".")
     just_compile = bool(args.just_compile)
@@ -585,6 +661,7 @@ def run_client_server(args: Args) -> int:
             clear=args.clear,
             no_platformio=no_platformio,
             app=app,
+            background_update=background_update,
         )
     except KeyboardInterrupt:
         return 1
