@@ -15,6 +15,7 @@ from fastled.docker_manager import (
     RunningContainer,
     Volume,
 )
+from fastled.emoji_util import EMO
 from fastled.settings import DEFAULT_CONTAINER_NAME, IMAGE_NAME, SERVER_PORT
 from fastled.sketch import looks_like_fastled_repo
 from fastled.types import BuildMode, CompileResult, CompileServerError
@@ -76,7 +77,7 @@ class CompileServerImpl:
         # If we don't have fastled_src_dir (not in FastLED repo), disable libcompile
         if allow_libcompile and self.fastled_src_dir is None:
             print(
-                "⚠️  libfastled compilation disabled: volume source mapping not available"
+                f"{EMO('⚠️', 'WARNING:')}  libfastled compilation disabled: volume source mapping not available"
             )
             print("   (not running in FastLED repository)")
             allow_libcompile = False
@@ -337,3 +338,14 @@ class CompileServerImpl:
         self.docker.suspend_container(self.container_name)
         self._port = 0
         print("Compile server stopped")
+
+    def get_emsdk_headers(self, filepath: Path) -> None:
+        """Get EMSDK headers ZIP data from the server and save to filepath."""
+        from fastled.util import download_emsdk_headers
+
+        if not self._port:
+            raise RuntimeError("Server has not been started yet")
+        if not self.ping():
+            raise RuntimeError("Server is not running")
+
+        download_emsdk_headers(self.url(), filepath)
