@@ -34,13 +34,16 @@ FastLED WASM Compiler - Useful options:
   --quick               Build in quick mode (default)
   --profile             Enable profiling the C++ build system
   --update              Update the docker image for the wasm compiler
+  --background-update   Update the docker image in the background after compilation
   --purge               Remove all FastLED containers and images
+  --emsdk-headers <path> Export EMSDK headers ZIP to specified path
   --version             Show version information
   --help                Show detailed help
 Examples:
   fastled (will auto detect the sketch directory and prompt you)
   fastled my_sketch
   fastled my_sketch --web (compiles using the web compiler only)
+  fastled my_sketch --background-update (compiles and updates docker image in background)
   fastled --init Blink (initializes a new sketch directory with the Blink example)
   fastled --server (runs the compiler server in the current directory)
 
@@ -132,6 +135,11 @@ def parse_args() -> Args:
         action="store_true",
         help="Update the wasm compiler (if necessary) before running",
     )
+    parser.add_argument(
+        "--background-update",
+        action="store_true",
+        help="Update the docker image in the background after compilation (user doesn't have to wait)",
+    )
     parser.add_argument(
         "--localhost",
         "--local",
@@ -163,6 +171,30 @@ def parse_args() -> Args:
         help="Remove all FastLED containers and images",
     )
 
+    parser.add_argument(
+        "--install",
+        action="store_true",
+        help="Install FastLED development environment with VSCode configuration and Auto Debug extension",
+    )
+
+    parser.add_argument(
+        "--dry-run",
+        action="store_true",
+        help="Run in dry-run mode (simulate actions without making changes)",
+    )
+
+    parser.add_argument(
+        "--no-interactive",
+        action="store_true",
+        help="Run in non-interactive mode (fail instead of prompting for input)",
+    )
+    parser.add_argument(
+        "--emsdk-headers",
+        type=str,
+        default=None,
+        help="Export EMSDK headers ZIP to specified path",
+    )
+
     build_mode = parser.add_mutually_exclusive_group()
     build_mode.add_argument("--debug", action="store_true", help="Build in debug mode")
     build_mode.add_argument(
@@ -178,6 +210,38 @@ def parse_args() -> Args:
 
     args = parser.parse_args()
 
+    # Handle --emsdk-headers early before other processing
+    if args.emsdk_headers:
+        from fastled.header_dump import dump_emsdk_headers
+
+        out_path = args.emsdk_headers
+        dump_emsdk_headers(out_path)
+        sys.exit(0)
+
+    # Auto-enable app mode if debug is used and Playwright cache exists
+    if args.debug and not args.app:
+        playwright_dir = Path.home() / ".fastled" / "playwright"
+        if playwright_dir.exists() and any(playwright_dir.iterdir()):
+            from fastled.emoji_util import EMO
+
+            print(
+                f"{EMO('⚠️', 'WARNING:')} Debug mode detected with Playwright installed - automatically enabling app mode"
+            )
+            args.app = True
+        elif not args.no_interactive:
+            # Prompt user to install Playwright only if not in no-interactive mode
+            answer = (
+                input("Would you like to install the FastLED debugger? [y/n] ")
+                .strip()
+                .lower()
+            )
+            if answer in ["y", "yes"]:
+                print(
+                    "📦 To install Playwright, run: pip install playwright && python -m playwright install"
+                )
+                print("Then run your command again with --app flag")
+                sys.exit(0)
+
     # TODO: propagate the library.
     # from fastled.docker_manager import force_remove_previous
 
@@ -197,6 +261,11 @@ def parse_args() -> Args:
     #     print(msg)
     #     set_ramdisk_size(args.ram_disk_size)
 
+    # Handle --install early before other processing
+    if args.install:
+        # Don't process other arguments when --install is used
+        return Args.from_namespace(args)
+
     if args.purge:
         from fastled.docker_manager import DockerManager
 
