@@ -14,6 +14,8 @@ import warnings
 from pathlib import Path
 from typing import Any
 
+from fastled.playwright.resize_tracking import ResizeTracker
+
 # Set custom Playwright browser installation path
 PLAYWRIGHT_DIR = Path.home() / ".fastled" / "playwright"
 PLAYWRIGHT_DIR.mkdir(parents=True, exist_ok=True)
@@ -69,6 +71,7 @@ class PlaywrightBrowser:
         self.playwright: Any = None
         self._should_exit = asyncio.Event()
         self._extensions_dir: Path | None = None
+        self.resize_tracker: ResizeTracker | None = None
 
         # Initialize extensions if enabled
         if self.enable_extensions:
@@ -278,118 +281,98 @@ class PlaywrightBrowser:
             "[PYTHON] Setting up browser window tracking with viewport-only adjustment"
         )
 
+        # Create resize tracker instance
+        self.resize_tracker = ResizeTracker(self.page)
+
         # Start polling loop that tracks browser window changes and adjusts viewport only
         asyncio.create_task(self._track_browser_adjust_viewport())
 
     async def _track_browser_adjust_viewport(self) -> None:
         """Track browser window changes and adjust viewport accordingly.
 
-        This method polls for changes in the browser window size and adjusts
-        the viewport size to match, maintaining the browser window dimensions
-        while ensuring the content area matches the sketch requirements.
+        This method polls for changes in the browser window size using the
+        ResizeTracker and handles any errors that occur.
         """
-        last_viewport = None
-        consecutive_same_count = 0
-        max_consecutive_same = 5
+        if self.resize_tracker is None:
+            print("[PYTHON] Cannot start tracking: resize_tracker is None")
+            return
 
         while not self._should_exit.is_set():
             try:
-                await asyncio.sleep(0.5)  # Poll every 500ms
+                # Wait between polls
+                await asyncio.sleep(0.25)  # Poll every 250ms
 
-                if self.page is None:
-                    continue
+                # Check if page is still alive
+                if self.page is None or self.page.is_closed():
+                    print("[PYTHON] Page closed, signaling exit")
+                    self._should_exit.set()
+                    return
 
-                # Get current viewport size
-                current_viewport = await self.page.evaluate(
-                    """
-                    () => ({
-                        width: window.innerWidth,
-                        height: window.innerHeight
-                    })
-                """
-                )
+                # Update resize tracking
+                result = await self.resize_tracker.update()
 
-                # Check if viewport changed
-                if current_viewport != last_viewport:
-                    last_viewport = current_viewport
-                    consecutive_same_count = 0
+                if result is not None:
+                    assert isinstance(result, Exception)
+                    # An exception occurred in resize tracking
+                    error_message = str(result)
+                    warnings.warn(f"[PYTHON] Error in resize tracking: {error_message}")
 
-                    print(
-                        f"[PYTHON] Viewport: {current_viewport['width']}x{current_viewport['height']}"
+                    # Be EXTREMELY conservative about browser close detection
+                    # Only trigger shutdown on very specific errors that definitively indicate browser closure
+                    browser_definitely_closed = any(
+                        phrase in error_message.lower()
+                        for phrase in [
+                            "browser has been closed",
+                            "target closed",
+                            "connection closed",
+                            "target page, probably because the page has been closed",
+                            "page has been closed",
+                            "browser context has been closed",
+                        ]
                     )
 
-                    # Try to get window outer dimensions for context
+                    # Also check actual browser state before deciding to shut down
+                    browser_state_indicates_closed = False
                     try:
-                        window_info = await self.page.evaluate(
-                            """
-                        () => ({
-                            outerWidth: window.outerWidth,
-                            outerHeight: window.outerHeight,
-                            screenX: window.screenX,
-                            screenY: window.screenY
-                        })
-                    """
-                        )
-
-                        print(
-                            f"[PYTHON] Window: {window_info['outerWidth']}x{window_info['outerHeight']} at ({window_info['screenX']}, {window_info['screenY']})"
-                        )
-
+                        if self.browser and hasattr(self.browser, "is_closed"):
+                            browser_state_indicates_closed = self.browser.is_closed()
+                        elif self.context and hasattr(self.context, "closed"):
+                            browser_state_indicates_closed = self.context.closed
                     except Exception:
-                        pass
+                        # If we can't check the state, don't assume it's closed
+                        warnings.warn(
+                            f"[PYTHON] Could not check browser state: {result}. Assuming browser is not closed."
+                        )
+                        browser_state_indicates_closed = False
 
-                else:
-                    consecutive_same_count += 1
-                    if consecutive_same_count >= max_consecutive_same:
-                        # Viewport hasn't changed for a while, reduce polling frequency
-                        await asyncio.sleep(2.0)
-                        consecutive_same_count = 0
-
-                # Get the browser window information periodically
-                try:
-                    browser_info = await self.page.evaluate(
-                        """
-                        () => {
-                            return {
-                                userAgent: navigator.userAgent,
-                                platform: navigator.platform,
-                                cookieEnabled: navigator.cookieEnabled,
-                                language: navigator.language
-                            };
-                        }
-                    """
-                    )
-
-                    if browser_info:
-                        pass  # We have browser info, but don't need to print it constantly
+                    if browser_definitely_closed or browser_state_indicates_closed:
+                        if browser_definitely_closed:
+                            print(
+                                f'[PYTHON] Browser has been closed because "{error_message}" matched one of the error phrases or browser state indicates closed, shutting down gracefully...'
+                            )
+                        elif browser_state_indicates_closed:
+                            print(
+                                "[PYTHON] Browser state indicates closed, shutting down gracefully..."
+                            )
+                        self._should_exit.set()
+                        return
                     else:
-                        print("[PYTHON] Could not get browser window info")
-
-                except Exception as e:
-                    print(f"[PYTHON] Could not get browser info: {e}")
+                        # For other errors, just log and continue - don't shut down
+                        print(
+                            f"[PYTHON] Recoverable error in resize tracking: {result}"
+                        )
+                        # Add a small delay to prevent tight error loops
+                        await asyncio.sleep(1.0)
 
             except Exception as e:
                 error_message = str(e)
-                # Check if the error indicates the browser/page has been closed
-                if any(
-                    keyword in error_message.lower()
-                    for keyword in [
-                        "target page",
-                        "context",
-                        "browser has been closed",
-                        "page.evaluate",
-                        "browser closed",
-                        "target closed",
-                    ]
-                ):
-                    print(
-                        "[PYTHON] Browser has been closed, shutting down gracefully..."
-                    )
-                    self._should_exit.set()
-                    break
-                else:
-                    print(f"[PYTHON] Error in browser tracking: {e}")
-                    continue
+                warnings.warn(
+                    f"[PYTHON] Unexpected error in browser tracking loop: {error_message}"
+                )
+                # Add a small delay to prevent tight error loops
+                await asyncio.sleep(1.0)
+
+        warnings.warn("[PYTHON] Browser tracking loop exited.")
 
     async def wait_for_close(self) -> None:
         """Wait for the browser to be closed."""
@@ -413,21 +396,52 @@ class PlaywrightBrowser:
         # Signal all tracking loops to exit
         self._should_exit.set()
 
-        if self.page:
-            await self.page.close()
-            self.page = None
+        try:
+            # Clean up resize tracker
+            self.resize_tracker = None
 
-        if self.context:
-            await self.context.close()
-            self.context = None
+            if self.page:
+                await self.page.close()
+                self.page = None
 
-        if self.browser:
-            await self.browser.close()
-            self.browser = None
+            if self.context:
+                await self.context.close()
+                self.context = None
 
-        if self.playwright:
-            await self.playwright.stop()
-            self.playwright = None
+            if self.browser:
+                await self.browser.close()
+                self.browser = None
+
+            if self.playwright:
+                # The playwright context manager may not have a stop() method in all versions
+                # Try stop() first, fall back to __aexit__ if needed
+                try:
+                    if hasattr(self.playwright, "stop"):
+                        await self.playwright.stop()
+                    else:
+                        # For async context managers, use __aexit__
+                        await self.playwright.__aexit__(None, None, None)
+                except Exception as stop_error:
+                    print(
+                        f"[PYTHON] Warning: Could not properly stop playwright: {stop_error}"
+                    )
+                    # Try alternative cleanup methods
+                    try:
+                        if hasattr(self.playwright, "__aexit__"):
+                            await self.playwright.__aexit__(None, None, None)
+                    except Exception:
+                        pass  # Ignore secondary cleanup failures
+                finally:
+                    self.playwright = None
+        except KeyboardInterrupt:
+            print("[PYTHON] Keyboard interrupt detected, closing Playwright browser")
+            self._should_exit.set()
+            import _thread
+
+            _thread.interrupt_main()
+        except Exception as e:
+            print(f"[PYTHON] Error closing Playwright browser: {e}")
+            self._should_exit.set()
 
 
 def run_playwright_browser(
@@ -562,6 +576,11 @@ class PlaywrightBrowserProxy:
                     # Force exit the entire program
                     os._exit(0)
 
+            except KeyboardInterrupt:
+                print("[MAIN] Browser monitor interrupted by user")
+                import _thread
+
+                _thread.interrupt_main()
             except Exception as e:
                 print(f"[MAIN] Error monitoring browser process: {e}")
 
